<?php

namespace Drupal\aws_cloud\Access;

use Aws\Exception\AwsException;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\aws_cloud\Service\Iam\IamServiceInterface;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Access validation for Aws resource operations/forms.
 *
 * @package Drupal\aws_cloud\Access
 */
class AwsCloudAccess implements AwsCloudAccessInterface {

  use CloudContentEntityTrait;

  /**
   * EC2 service.
   *
   * @var \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface
   */
  protected $ec2Service;

  /**
   * IAM service.
   *
   * @var \Drupal\aws_cloud\Service\Iam\IamServiceInterface
   */
  protected $iamService;

  /**
   * AwsCloudAccess constructor.
   *
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The EC2 service.
   * @param \Drupal\aws_cloud\Service\Iam\IamServiceInterface $iam_service
   *   The IAM service.
   */
  public function __construct(Ec2ServiceInterface $ec2_service,
                              IamServiceInterface $iam_service) {
    $this->ec2Service = $ec2_service;
    $this->iamService = $iam_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): AwsCloudAccess {
    return new static(
      $container->get('aws_cloud.ec2'),
      $container->get('aws_cloud.iam')
    );
  }

  /**
   * Validate IAM permissions by invoking AWS APIs.
   *
   * @param string $perm
   *   Permission.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface|\Drupal\aws_cloud\Service\Iam\IamServiceInterface $service
   *   Service object, EC2 or IAM.
   * @param string $cloud_context
   *   Cloud context.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Access result.
   */
  private function validateIamPermissions(string $perm, $service, string $cloud_context): AccessResultInterface {

    // 1. Check arguments.
    if (empty($perm)
      || empty($cloud_context)) {
      return AccessResult::allowed();
    }

    // 2. Check if $service is Ec2ServiceInterface or IamServiceInterface.
    if (!($service instanceof Ec2ServiceInterface)
      && !($service instanceof IamServiceInterface)) {
      return AccessResult::allowed();
    }

    // 3. Check if $service::PERMISSION_TO_AWS_API[$perm] is valid or not.
    if (is_array($service::PERMISSION_TO_AWS_API)
      && !array_key_exists($perm, $service::PERMISSION_TO_AWS_API)) {
      return AccessResult::allowed();
    }

    // 4. Set $cloud_context.
    $service->setCloudContext($cloud_context);

    // 5. Validate IAM permissions for AWS APIs.
    $apis = (array) $service::PERMISSION_TO_AWS_API[$perm];
    foreach ($apis as $api) {
      try {
        $method = lcfirst($api);
        $service->$method(['DryRun' => TRUE]);
      }
      catch (AwsException $e) {
        // If permission is granted, the error code is DryRunOperation.
        // If not, the error code is one of permissionErrorCodes.
        $error_code = $e->getAwsErrorCode();

        if (in_array($error_code, $service::PERMISSION_ERROR_CODES, TRUE)) {
          $this->logger('aws_cloud')->error($e->getMessage());
          return AccessResult::forbidden();
        }
        // Other error code means an unexpected error occurs.
        // @FIXME This condition will be cleaned once AWS-service-related
        // constants are better defined.
        if ($service instanceof IamServiceInterface
          || $error_code !== Ec2ServiceInterface::DRY_RUN_OPERATION) {
          $this->logger('aws_cloud')->error($e->getMessage());
        }
      }
      catch (\Exception $e) {
        // Continue processing even if unexpected exceptions occur.
        $this->logger('aws_cloud')->error($e->getMessage());
      }
    }
    return AccessResult::allowed();
  }

  /**
   * {@inheritDoc}
   */
  public function access(string $cloud_context, AccountInterface $account, Route $route): AccessResultInterface {
    $perm = $route->getOption('perm');
    return $this->validateIamPermissions($perm, $this->ec2Service, $cloud_context)
      ->andIf($this->validateIamPermissions($perm, $this->iamService, $cloud_context));
  }

}
