<?php

namespace Drupal\aws_cloud\Controller\Ec2;

use Drupal\aws_cloud\Entity\Ec2\InstanceInterface;
use Drupal\aws_cloud\Service\CloudWatch\CloudWatchServiceInterface;
use Drupal\aws_cloud\Service\Ec2\Ec2OperationsServiceInterface;
use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\aws_cloud\Traits\AwsCloudEntityCheckTrait;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Service\CloudServiceInterface;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\cloud\Traits\CloudResourceTrait;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormState;
use Drupal\Core\Link;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Controller responsible for "update" URLs.
 *
 * This class is mainly responsible for
 * updating the AWS entities from URLs.
 */
class ApiController extends ControllerBase implements ApiControllerInterface {

  use AwsCloudEntityCheckTrait;
  use CloudContentEntityTrait;
  use CloudResourceTrait;

  /**
   * The AWS Cloud EC2 Service.
   *
   * @var \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface
   */
  private $ec2Service;

  /**
   * The AWS Cloud EC2 Operation Service.
   *
   * @var \Drupal\aws_cloud\Service\Ec2\Ec2OperationsServiceInterface
   */
  private $ec2OperationsService;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  private $renderer;

  /**
   * The AWS Cloud CloudWatch Service.
   *
   * @var \Drupal\aws_cloud\Service\CloudWatch\CloudWatchServiceInterface
   */
  private $cloudWatchService;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * Cloud Service interface.
   *
   * @var \Drupal\cloud\Service\CloudServiceInterface
   */
  protected $cloudService;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * ApiController constructor.
   *
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   Object for interfacing with AWS API.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2OperationsServiceInterface $ec2_operations_service
   *   The AWS Cloud EC2 Operations API service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Messenger Object.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack object.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\aws_cloud\Service\CloudWatch\CloudWatchServiceInterface $cloud_watch_service
   *   The AWS Cloud CloudWatch Service.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   Cloud service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(
    Ec2ServiceInterface $ec2_service,
    Ec2OperationsServiceInterface $ec2_operations_service,
    Messenger $messenger,
    RequestStack $request_stack,
    RendererInterface $renderer,
    CloudWatchServiceInterface $cloud_watch_service,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    CloudServiceInterface $cloud_service,
    EntityTypeManagerInterface $entity_type_manager,
    AccountInterface $current_user) {

    $this->ec2Service = $ec2_service;
    $this->ec2OperationsService = $ec2_operations_service;
    $this->messenger = $messenger;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->cloudWatchService = $cloud_watch_service;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->cloudService = $cloud_service;
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return ApiController
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('aws_cloud.ec2'),
      $container->get('aws_cloud.ec2_operations'),
      $container->get('messenger'),
      $container->get('request_stack'),
      $container->get('renderer'),
      $container->get('aws_cloud.cloud_watch'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('cloud'),
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirects to the cloud_config collection.
   */
  public function updateAll(): RedirectResponse {
    $regions = $this->requestStack->getCurrentRequest()->query->get('regions');
    if ($regions === NULL) {
      $this->messageUser($this->t('No region specified'), 'error');
    }
    else {
      $regions_array = explode(',', $regions);

      foreach ($regions_array ?: [] as $region) {
        $entity = $this->entityTypeManager()->getStorage('cloud_config')
          ->loadByProperties(
            [
              'cloud_context' => $region,
            ]);
        if ($entity) {
          aws_cloud_update_ec2_resources(array_shift($entity));
        }
      }

      $this->messageUser($this->t('Creating cloud service provider was performed successfully.'));
      drupal_flush_all_caches();
    }
    return $this->redirect('entity.cloud_config.collection');
  }

  /**
   * Update message after refresh resources.
   *
   * @param string $resource_type
   *   Entity type name.
   * @param bool $updated
   *   Updated statuses.
   */
  protected function updateMessages($resource_type, bool $updated): void {
    $labels = $this->getDisplayLabels($resource_type);
    if ($updated) {
      $this->messageUser($this->t('Updated @resource_name.', ['@resource_name' => $labels['plural']]));
    }
    else {
      $this->messageUser($this->t('Unable to update @resource_name.', ['@resource_name' => $labels['plural']]), 'error');
    }
    $this->cloudService->invalidateCacheTags();
  }

  /**
   * Update all instances in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to instance list of particular cloud region.
   */
  public function updateInstanceList($cloud_context): RedirectResponse {
    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateInstances();

    $this->updateMessages('aws_cloud_instance', $updated);

    return $this->redirect('view.aws_cloud_instance.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all instances of all cloud region.
   */
  public function updateAllInstanceList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_instance');
  }

  /**
   * Update all images in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to all image list.
   */
  public function updateImageList($cloud_context): RedirectResponse {
    $cloud_config_entities = $this->entityTypeManager()->getStorage('cloud_config')->loadByProperties(
      ['cloud_context' => [$cloud_context]]
    );

    if (!empty($cloud_config_entities)) {
      $cloud_config = reset($cloud_config_entities);
    }

    $account_id = $this->ec2Service->getAccountId($cloud_config_entities);

    if ($account_id) {
      $this->ec2Service->setCloudContext($cloud_context);
      $updated = $this->ec2Service->updateImages([
        'Owners' => [
          $account_id,
        ],
      ], TRUE);

      $this->updateMessages('aws_cloud_image', $updated);
    }
    else {
      $message = $this->t('AWS User ID is not specified.');
      $account = $this->currentUser();
      if ($account->hasPermission('edit cloud service providers')) {
        $message = Link::createFromRoute($message, 'entity.cloud_config.edit_form', ['cloud_config' => $cloud_config->id()])->toString();
      }
      $this->messageUser($message, 'error');
    }

    return $this->redirect('view.aws_cloud_image.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all images of all cloud region.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to all image list.
   */
  public function updateAllImageList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_image');
  }

  /**
   * Update all security groups in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to all security group list.
   */
  public function updateSecurityGroupList($cloud_context): RedirectResponse {
    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateSecurityGroups();

    $this->updateMessages('aws_cloud_security_group', $updated);

    return $this->redirect('view.aws_cloud_security_group.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all security groups of all cloud region.
   */
  public function updateAllSecurityGroupList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_security_group');
  }

  /**
   * Update all network interfaces in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to network interface list of particular cloud region.
   */
  public function updateNetworkInterfaceList($cloud_context): RedirectResponse {
    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateNetworkInterfaces();

    $this->updateMessages('aws_cloud_network_interface', $updated);

    return $this->redirect('view.aws_cloud_network_interface.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all network interfaces of all cloud region.
   */
  public function updateAllNetworkInterfaceList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_network_interface');
  }

  /**
   * Update all elastic ips in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to elastic ip list of particular cloud region.
   */
  public function updateElasticIpList($cloud_context): RedirectResponse {

    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateElasticIps();

    if ($updated === TRUE) {
      // Also update Network Interfaces.
      $updated = $this->ec2Service->updateNetworkInterfaces();
      if ($updated !== FALSE) {
        $this->messageUser($this->t('Updated Elastic IPs and Network Interfaces.'));
      }
      else {
        $this->messageUser(
          $this->t('Unable to update Network Interfaces while updating Elastic IPs.'),
          'error');
      }
      $this->cloudService->invalidateCacheTags();
    }
    else {
      $this->messageUser($this->t('Unable to update Elastic IPs.'), 'error');
    }

    return $this->redirect('view.aws_cloud_elastic_ip.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all elastic ips of all cloud region.
   */
  public function updateAllElasticIpList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_elastic_ip');
  }

  /**
   * Update all Key Pairs in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to keypair list of particular cloud region.
   */
  public function updateKeyPairList($cloud_context): RedirectResponse {

    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateKeyPairs();

    $this->updateMessages('aws_cloud_key_pair', $updated);
    return $this->redirect('view.aws_cloud_key_pair.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all Key Pairs of all cloud region.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to all instance list.
   */
  public function updateAllKeyPairList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_key_pair');
  }

  /**
   * Update all volumes in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to volume list of particular cloud region.
   */
  public function updateVolumeList($cloud_context): RedirectResponse {
    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateVolumes();

    $this->updateMessages('aws_cloud_volume', $updated);

    return $this->redirect('view.aws_cloud_volume.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all volumes of all cloud region.
   */
  public function updateAllVolumeList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_volume');
  }

  /**
   * Update all snapshots in particular cloud region.
   *
   * @param string $cloud_context
   *   Cloud context string.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect to snapshot list of particular cloud region.
   */
  public function updateSnapshotList($cloud_context): RedirectResponse {

    $this->ec2Service->setCloudContext($cloud_context);
    $updated = $this->ec2Service->updateSnapshots();

    $this->updateMessages('aws_cloud_snapshot', $updated);

    return $this->redirect('view.aws_cloud_snapshot.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * Update all snapshots of all cloud region.
   */
  public function updateAllSnapshotList(): RedirectResponse {
    return $this->ec2Service->updateAllResourceList('aws_cloud_snapshot');
  }

  /**
   * {@inheritdoc}
   */
  public function listInstanceCallback($cloud_context): Response {
    return $this->getViewResponse('aws_cloud_instance');
  }

  /**
   * {@inheritdoc}
   */
  public function listImageCallback($cloud_context): Response {
    return $this->getViewResponse('aws_cloud_image');
  }

  /**
   * {@inheritdoc}
   */
  public function listSnapshotCallback($cloud_context): Response {
    return $this->getViewResponse('aws_cloud_snapshot');
  }

  /**
   * {@inheritdoc}
   */
  public function listVolumeCallback($cloud_context): Response {
    return $this->getViewResponse('aws_cloud_volume');
  }

  /**
   * {@inheritdoc}
   */
  public function searchImages($cloud_context): Response {
    $this->ec2Service->setCloudContext($cloud_context);
    $name = $this->requestStack->getCurrentRequest()->query->get('q');
    $result = $this->ec2Service->describeImages([
      'Filters' => [
        [
          'Name' => 'name',
          'Values' => [$name],
        ],
        [
          'Name' => 'state',
          'Values' => ['available'],
        ],
      ],
    ]);

    $limit = 50;
    $count = 0;
    $images = [];
    foreach ($result['Images'] ?: [] as $image) {
      $images[] = [
        'name' => $image['Name'],
        'id' => $image['Name'],
        'created' => strtotime($image['CreationDate']),
      ];
      if ($count++ >= $limit) {
        break;
      }
    }

    // Sort by latest.
    usort($images, static function ($a, $b) {
      return ($a['created'] < $b['created']) ? 1 : -1;
    });

    return new Response(json_encode($images));
  }

  /**
   * {@inheritdoc}
   */
  public function getInstanceMetrics($cloud_context, InstanceInterface $aws_cloud_instance): JsonResponse {
    $this->cloudWatchService->setCloudContext($cloud_context);
    $metric_names = [
      'cpu' => 'CPUUtilization',
      'network_in' => 'NetworkIn',
      'network_out' => 'NetworkOut',
      'disk_read' => 'DiskReadBytes',
      'disk_write' => 'DiskWriteBytes',
      'disk_read_operation' => 'DiskReadOps',
      'disk_write_operation' => 'DiskWriteOps',
    ];
    $queries = [];
    foreach ($metric_names as $key => $name) {
      $queries[] = [
        'Id' => $key,
        'MetricStat' => [
          'Metric' => [
            'Namespace' => 'AWS/EC2',
            'MetricName' => $name,
            'Dimensions' => [
              [
                'Name' => 'InstanceId',
                'Value' => $aws_cloud_instance->getInstanceId(),
              ],
            ],
          ],
          'Period' => 300,
          'Stat' => 'Average',
        ],
      ];
    }

    $result = $this->cloudWatchService->getMetricData([
      'StartTime' => strtotime('-1 days'),
      'EndTime' => strtotime('now'),
      'MetricDataQueries' => $queries,
    ]);

    $data = [];
    foreach (array_keys($metric_names) as $index => $key) {
      $timestamps = [];
      foreach ($result['MetricDataResults'][$index]['Timestamps'] as $timestamp) {
        $timestamps[] = $timestamp->__toString();
      }

      $data[$key] = [
        'timestamps' => $timestamps,
        'values' => $result['MetricDataResults'][$index]['Values'],
      ];

      if ($key === 'network_in' || $key === 'network_out') {
        // Convert Byte to MB.
        $data[$key]['values'] = array_map(function ($value) {
          return $value / 1024 / 1024;
        }, $data[$key]['values']);
      }
    }

    return new JsonResponse($data);
  }

  /**
   * Helper method to get views output.
   *
   * @param string $view_id
   *   The ID of list view.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response of list view.
   */
  private function getViewResponse($view_id): Response {
    $view = Views::getView($view_id);

    // Set the display machine name.
    $view->setDisplay('list');

    // Render the view as html, and return it as a response object.
    $build = $view->executeDisplay();
    return new Response($this->renderer->render($build));
  }

  /**
   * Helper method to add messages for the end user.
   *
   * @param string $message
   *   The message.
   * @param string $type
   *   The message type: error or message.
   */
  private function messageUser($message, $type = 'message'): void {
    switch ($type) {
      case 'error':
        $this->messenger->addError($message);
        break;

      case 'message':
        $this->messenger->addStatus($message);
      default:
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function updateAwsResource(string $cloud_context, string $entity_type_id, string $entity_id): JsonResponse {
    $entity = $this->entityTypeManager
      ->getStorage($entity_type_id)
      ->load($entity_id);

    $this->ec2Service->setCloudContext($cloud_context);
    $this->ec2Service->updateSingleEntity($entity, FALSE, TRUE);

    return new JsonResponse(['result' => 'OK']);
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityCount(string $cloud_context, string $entity_type_id): JsonResponse {
    $params = [
      'cloud_context' => $cloud_context,
    ];
    // @todo Set the exact permissions after the research on permissions have completed
    $count = $this->getResourceCount($entity_type_id, "list {$this->convertUnderscoreToWhitespace($entity_type_id)}", $params);

    return new JsonResponse(['count' => $count]);
  }

  /**
   * Get entity data of AWS Cloud instance for entity ID.
   *
   * @param string $aws_cloud_instance
   *   The entity ID of AWS Cloud instance string.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity data.
   */
  private function getInstanceEntity(string $aws_cloud_instance) {
    $entity = $this->entityTypeManager
      ->getStorage('aws_cloud_instance')
      ->load($aws_cloud_instance);

    if (!$this->updateEntitySubmitForm(new FormState(), $entity)) {
      return NULL;
    }
    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getConsoleOutputFromInstance(string $cloud_context, string $aws_cloud_instance): JsonResponse {
    // Get entity data.
    $entity = $this->getInstanceEntity($aws_cloud_instance);
    if (empty($entity)) {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'The instance has already been deleted.',
      ], 404);
    }

    // Execute AWS API.
    $result = $this->ec2OperationsService->getConsoleOutputFromInstance($entity);

    return !empty($result)
      ? new JsonResponse([
        'result' => 'OK',
        'log' => $result,
      ], 200)
      : new JsonResponse([
        'result' => 'NG',
        'reason' => 'Internal Server Error',
      ], 500);
  }

  /**
   * {@inheritdoc}
   */
  public function operateInstance(Request $request, string $cloud_context, string $aws_cloud_instance, string $command): JsonResponse {
    // Validation check.
    $operations_commandList = [
      'start',
      'stop',
      'reboot',
      'delete',
      'create_image',
      'associate_elastic_ip',
      'edit',
    ];
    if (!in_array($command, $operations_commandList, TRUE)) {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'Bad Request',
      ], 400);
    }

    // Get entity data.
    $entity = $this->getInstanceEntity($aws_cloud_instance);
    if (empty($entity)) {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'The instance has already been deleted.',
      ], 404);
    }

    // Execute AWS API and save entity data.
    $result = FALSE;
    $parameter = NULL;
    switch ($command) {
      case 'create_image':
        /* JSON format in request body.
         * {
         *   imageName: string;
         *   noRebootFlg: boolean;
         * }
         */
        $parameter = json_decode($request->getContent(), TRUE);
        $command = 'createImageFrom';
        break;

      case 'associate_elastic_ip':
        /* JSON format in request body.
         * {
         *   allocationId: string;
         *   networkInterfaceId: string;
         * }
         */
        $parameter = json_decode($request->getContent(), TRUE);
        $command = 'associateElasticIpFor';
        break;

      case 'edit':
        /* JSON format in request body.
         * {
         *   // UNIX second. (int)
         *   termination_timestamp: number | null;
         *   security_groups: string[];
         *   // _weight……element's weihht (number string)
         *   // item_key……item's key (text string)
         *   // item_value……item's value (text string)
         *   tags: {_weight: string, item_key: string, item_value: string}[];
         *   schedule: string | null;
         *   instance_type: string | null;
         *   termination_protection: boolean | null;
         *   user_data: string | null;
         *   // Number string ID or blank string. (e.g. '13', '')
         *   new_elastic_ip: string | null;
         *   current_allocation_id: string | null;
         *   current_association_id: string | null;
         * }
         */
        $parameter = json_decode($request->getContent(), TRUE);
        break;

      default:
        break;
    }
    $method_name = "${command}Instance";
    if (method_exists($this->ec2OperationsService, $method_name)) {
      $result = $this->ec2OperationsService->$method_name($entity, $parameter);
    }

    return !empty($result)
      ? new JsonResponse([
        'result' => 'OK',
      ], 200)
      : new JsonResponse([
        'result' => 'NG',
        'reason' => 'Internal Server Error',
      ], 500);
  }

}
