<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the ElasticIp entity create form.
 *
 * @ingroup aws_cloud
 */
class ElasticIpCreateForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {
    /** @var \Drupal\aws_cloud\Entity\Ec2\ElasticIp $entity */
    $form = parent::buildForm($form, $form_state);

    $this->ec2Service->setCloudContext($cloud_context);

    $entity = $this->entity;

    $weight = -50;

    $form['elastic_ip'] = [
      '#type' => 'details',
      '#title' => $this->t('@title', ['@title' => $entity->getEntityType()->getSingularLabel()]),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['elastic_ip']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => TRUE,
    ];

    $form['elastic_ip']['domain'] = [
      '#type'          => 'select',
      '#options'       => [
        'standard' => $this->t('Standard'),
        'vpc' => $this->t('VPC'),
      ],
      '#title'         => $this->t('Domain (Standard | VPC)'),
      '#default_value' => 'standard',
      '#required'      => TRUE,
    ];

    $options = $this->getNetworkBorderGroupOptions($cloud_context);
    if (!empty($options)) {
      $form['elastic_ip']['network_border_group'] = [
        '#type'          => 'select',
        '#options'       => $this->getNetworkBorderGroupOptions($cloud_context),
        '#title'         => $this->t('Network Border Group'),
        '#required'      => TRUE,
      ];
    }

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    return $form;
  }

  /**
   * Get options for the select element network border group.
   *
   * @param string $cloud_context
   *   The cloud context.
   *
   * @return array
   *   The options.
   */
  private function getNetworkBorderGroupOptions($cloud_context): array {
    $options = [];

    $this->ec2Service->setCloudContext($cloud_context);
    $result = $this->ec2Service->describeAvailabilityZones();
    $group_zones = [];

    foreach ($result['AvailabilityZones'] ?: [] as $zone) {
      // Do nothing if there is no NetworkBorderGroup.
      if (empty($zone['NetworkBorderGroup'])) {
        return $options;
      }
      $group = $zone['NetworkBorderGroup'];
      if (empty($group_zones[$group])) {
        $group_zones[$group] = [];
      }
      $group_zones[$group][] = $zone['ZoneName'];
    }

    foreach ($group_zones as $group => $zones) {
      $options[$group] = sprintf('%s (%s)', $group, implode(', ', $zones));
    }

    return $options;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $params = [
      'Domain' => $entity->getDomain(),
    ];

    if (!empty($entity->getNetworkBorderGroup())) {
      $params['NetworkBorderGroup'] = $entity->getNetworkBorderGroup();
    }

    $result = $this->ec2Service->allocateAddress($params);

    $public_ip = $result['PublicIp'] ?? $result['CarrierIp'];
    $elastic_ip_type = !empty($result['PublicIp']) ? 'Public IP' : 'Carrier IP';
    if (!empty($public_ip)
    && $entity->setPublicIp($public_ip)
    && $entity->setAllocationId($result['AllocationId'])
    && $entity->setDomain($result['Domain'])
    && $entity->setElasticIpType($elastic_ip_type)
    && $entity->save()) {

      if (!empty($result['NetworkBorderGroup'])) {
        $entity->setNetworkBorderGroup($result['NetworkBorderGroup']);
        $entity->save();
      }

      // Update the entity name.
      $this->updateNameAndCreatedByTags($entity, $entity->getAllocationId());

      $this->processOperationStatus($entity, 'created');
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSaveEvent($entity);
    }
    else {
      $this->processOperationErrorStatus($entity, 'created');
    }

    $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);
  }

}
