<?php

namespace Drupal\aws_cloud\Plugin\Block;

use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Traits\CloudResourceTrait;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderableInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block displaying system resources.
 *
 * @Block(
 *   id = "aws_cloud_resources_block",
 *   admin_label = @Translation("AWS Resources"),
 *   category = @Translation("AWS Cloud")
 * )
 */
class AwsCloudResourcesBlock extends BlockBase implements ContainerFactoryPluginInterface {

  use CloudResourceTrait;

  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The cloud config plugin manager.
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * The route match object.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Creates a ResourcesBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud config plugin manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match object.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user,
    EntityTypeManagerInterface $entity_type_manager,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    RouteMatchInterface $route_match
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $form['cloud_context'] = [
      '#type' => 'select',
      '#title' => $this->t('Cloud Service Provider'),
      '#description' => $this->t('Select cloud service provider.'),
      '#options' => $this->getCloudConfigs($this->t('All AWS Cloud regions'), 'aws_cloud'),
      '#default_value' => $config['cloud_context'] ?? '',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['cloud_context'] = $form_state->getValue('cloud_context');
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'cloud_context' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $cloud_configs = $this->getCloudConfigs($this->t('All AWS Cloud regions'), 'aws_cloud');
    $cloud_context = $this->configuration['cloud_context'];
    $cloud_context_name = empty($cloud_context)
      ? 'All AWS Cloud regions'
      : $cloud_configs[$cloud_context];
    $build = [];
    $build['resources'] = [
      '#type' => 'details',
      '#title' => $this->t('Resources'),
      '#open' => TRUE,
    ];

    $build['resources']['description'] = [
      '#markup' => $this->t('You are using the following AWS resources in %cloud_context_name:',
        ['%cloud_context_name' => $cloud_context_name]
      ),
    ];
    $build['resources']['resource_table'] = $this->buildResourceTable();

    return $build;
  }

  /**
   * Build a resource HTML table.
   *
   * @return array
   *   Table render array.
   */
  private function buildResourceTable(): array {
    $resources = [
      'aws_cloud_instance' => [
        'view any aws cloud instance',
        [],
      ],
      'aws_cloud_image' => [
        'view any aws cloud image',
        [],
      ],
      'aws_cloud_security_group' => [
        'view any aws cloud security group',
        [],
      ],
      'aws_cloud_elastic_ip' => [
        'view any aws cloud elastic ip',
        [],
      ],
      'aws_cloud_network_interface' => [
        'view any aws cloud network interface',
        [],
      ],
      'aws_cloud_key_pair' => [
        'view any aws cloud key pair',
        [],
      ],
      'aws_cloud_volume' => [
        'view any aws cloud volume',
        [],
      ],
      'aws_cloud_snapshot' => [
        'view any aws cloud snapshot',
        [],
      ],
      'aws_cloud_vpc' => [
        'view any aws cloud vpc',
        [],
      ],
      'aws_cloud_subnet' => [
        'view any aws cloud subnet',
        [],
      ],
      'aws_cloud_vpc_peering_connection' => [
        'view any aws cloud vpc peering connection',
        [],
      ],
      'aws_cloud_internet_gateway' => [
        'view any aws cloud internet gateway',
        [],
      ],
      'aws_cloud_carrier_gateway' => [
        'view any aws cloud carrier gateway',
        [],
      ],
      'aws_cloud_transit_gateway' => [
        'view any aws cloud transit gateway',
        [],
      ],
      'instance_type_prices' => [
        'Instance Type Prices',
      ],
    ];

    $rows = $this->buildResourceTableRows($resources);

    return [
      '#type' => 'table',
      '#rows' => $rows,
    ];
  }

  /**
   * Build a resource table row.
   *
   * @param array $resources
   *   Entity array to build the row.
   *
   * @return array
   *   Fully built rows.
   */
  protected function buildResourceTableRows(array $resources): array {
    $rows = [];
    $data = [];
    $i = 0;
    $index = 0;

    foreach ($resources ?: [] as $key => $values) {
      $data[] = $key === 'instance_type_prices'
        ? $this->getInstanceTypePricingLink($key, $values[0])
        : $this->getResourceLink($key, $values[0], $values[1]);

      // Skip if $data doesn't have any value.
      if (empty($data[count($data) - 1])) {
        continue;
      }

      $rows[$index] = [
        'no_striping' => TRUE,
        'data' => $data,
      ];
      if ($i++ % 2) {
        $index++;
        $data = [];
      }
    }
    return $rows;
  }

  /**
   * Generate AWS resource link.
   *
   * @param string $resource_type
   *   The resource type.
   * @param string $permission
   *   The getResourceCount permission.
   * @param array $params
   *   The getResourceCount params.
   *
   * @return \Drupal\Core\Render\RenderableInterface
   *   The AWS resource link.
   */
  protected function getResourceLink($resource_type, $permission, array $params = []): RenderableInterface {
    // Fetch the labels.
    $labels = $this->getDisplayLabels($resource_type);
    $cloud_context = $this->configuration['cloud_context'];

    if (!empty($cloud_context)) {
      $route_name = "view.${resource_type}.list";
      $params = [
        'cloud_context' => $cloud_context,
      ];
    }
    else {
      $route_name = "view.${resource_type}.all";
    }

    return Link::createFromRoute(
      $this->formatPlural(
        $this->getResourceCount($resource_type, $permission, $params),
        '1 @label',
        '@count @plural_label',
        [
          '@label' => $labels['singular'] ?? $resource_type,
          '@plural_label' => $labels['plural'] ?? $resource_type,
        ]
      ),
      $route_name,
      $params
    );
  }

  /**
   * Generate AWS Instance Type Pricing link.
   *
   * @param string $resource_type
   *   The resource type.
   * @param string $label
   *   The link label.
   *
   * @return array
   *   The AWS resource link.
   */
  private function getInstanceTypePricingLink($resource_type, $label): array {

    // Return immediately if $cloud_context is empty.
    $cloud_context = $this->configuration['cloud_context'];
    if (empty($cloud_context)) {
      return [];
    }
    $instance = count(aws_cloud_get_instance_types($cloud_context));
    $pricing_label = "$instance $label";
    $link = Link::createFromRoute($pricing_label, "aws_cloud.${resource_type}",
      ['cloud_context' => $cloud_context]
    );

    return $link;
  }

}
