<?php

namespace Drupal\aws_cloud\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a block displaying long running instances.
 *
 * @Block(
 *   id = "aws_cloud_long_running_instances_block",
 *   admin_label = @Translation("Long Running Instances"),
 *   category = @Translation("AWS Cloud")
 * )
 */
class LongRunningInstancesBlock extends BlockBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Creates a LongRunningInstancesBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user,
    RequestStack $request_stack
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    return $this->buildInstanceList();
  }

  /**
   * Build a list of long running instances.
   *
   * @return array
   *   Array of instance URLs.
   */
  private function buildInstanceList(): array {
    $build = [];
    $build['instances'] = [
      '#type' => 'details',
      '#title' => $this->t('Long Running Instances'),
      '#open' => TRUE,
    ];

    $instances = $this->getLongRunningInstances();
    if (count($instances)) {
      $rows = [];
      foreach ($instances ?: [] as $instance) {
        $days_running = $instance->daysRunning();
        $days_format_plural = $this->formatPlural($days_running, '1 day', '@count days');

        $rows[] = [
          'name' => $instance->toLink($instance->getName()),
          'days' => $days_format_plural,
        ];
      }

      $unused_days = $this->configFactory->get('aws_cloud.settings')->get('aws_cloud_long_running_instance_notification_criteria');
      $build['instances'][] = [
        '#markup' => $this->t('The following instances have been running for more than %num days', ['%num' => $unused_days]),
      ];

      $headers = [
        ['data' => $this->t('Name'), 'field' => 'name'],
        ['data' => $this->t('Days'), 'field' => 'days'],
      ];

      $sort = $this->request->get('sort');
      $order = $this->request->get('order');

      $order_field = NULL;
      foreach ($headers ?: [] as $header) {
        if ($header['data']->render() === $order) {
          $order_field = $header['field'];
        }
      }

      // Get sort and order parameters.
      if (empty($sort)) {
        $sort = 'asc';
      }
      if (empty($order_field)) {
        $order_field = 'name';
      }

      // Sort data.
      usort($rows, static function ($a, $b) use ($sort, $order_field) {
        $result = 1;

        if ($order_field === 'name') {
          $result = $a[$order_field]->getText() < $b[$order_field]->getText() ? -1 : 1;
        }

        if ($order_field === 'days') {
          $a_days = explode(' ', $a[$order_field])[0];
          $b_days = explode(' ', $b[$order_field])[0];
          $result = $a_days < $b_days ? -1 : 1;
        }

        if ($sort === 'desc') {
          $result *= -1;
        }

        return $result;
      });

      $build['instances'][] = [
        '#theme' => 'table',
        '#header' => $headers,
        '#rows' => $rows,
      ];
    }
    else {
      $build['instances'][] = [
        '#markup' => $this->t('Great job! You have no long running instances.'),
      ];
    }
    return $build;
  }

  /**
   * Get a list of long running instances.
   *
   * @return array
   *   Array of long running instances.
   */
  private function getLongRunningInstances(): array {
    $instances = aws_cloud_get_long_running_instances();
    if (!$this->currentUser->hasPermission('view any aws cloud instance')) {
      foreach ($instances ?: [] as $key => $instance) {
        if ($instance->getOwnerId() !== $this->currentUser->id()) {
          unset($instances[$key]);
        }
      }
    }
    return $instances;
  }

  /**
   * The maximum age for which this object may be cached.
   *
   * @return int
   *   The maximum time in seconds that this object may be cached.
   */
  public function getCacheMaxAge(): int {
    return 0;
  }

}
