<?php

namespace Drupal\aws_cloud\Service\Ec2;

use Drupal\aws_cloud\Form\Ec2\InstanceEditForm;
use Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface;
use Drupal\aws_cloud\entity\ec2\ImageInterface;
use Drupal\aws_cloud\Entity\Ec2\Instance;
use Drupal\aws_cloud\Entity\Ec2\InstanceInterface;
use Drupal\cloud\Service\CloudServiceBase;
use Drupal\cloud\Service\CloudServiceInterface;
use Drupal\Core\Entity\EntityDeleteFormTrait;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;

/**
 * Entity update methods for REST API processing.
 */
class Ec2OperationsService extends CloudServiceBase implements Ec2OperationsServiceInterface {

  use EntityDeleteFormTrait;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The AWS Cloud EC2 Service.
   *
   * @var \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface
   */
  protected $ec2Service;

  /**
   * The entity.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $entity;

  /**
   * Ec2OperationsService constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2_service
   *   The AWS Cloud EC2 Service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager,
    Ec2ServiceInterface $ec2_service) {

    // The parent constructor takes care of $this->messenger object.
    parent::__construct();

    // Setup the entity type manager for querying entities.
    $this->entityTypeManager = $entity_type_manager;
    $this->ec2Service = $ec2_service;
  }

  /**
   * {@inheritdoc}
   */
  public function startInstance(InstanceInterface $entity): bool {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->startInstances([
      'InstanceIds' => [
        $entity->getInstanceId(),
      ],
    ]);

    if (empty($result)) {
      $this->processOperationErrorStatus($entity, 'started');
      return FALSE;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'started remotely');
      return TRUE;
    }

    try {
      $current_state = $result['StartingInstances'][0]['CurrentState']['Name'];
      $instance = $this->entityTypeManager
        ->getStorage($entity->getEntityTypeId())
        ->load($entity->id());
      $instance->setInstanceState($current_state);
      $instance->save();

      $this->processOperationStatus($instance, 'started');
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSubmitEvent($entity);
      return TRUE;
    }
    catch (\Exception $e) {
      $this->handleException($e);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function stopInstance(InstanceInterface $entity): bool {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->stopInstances([
      'InstanceIds' => [
        $entity->getInstanceId(),
      ],
    ]);

    if (empty($result)) {
      $this->processOperationErrorStatus($entity, 'stopped');
      return FALSE;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'stopped remotely');
      return TRUE;
    }

    try {
      $current_state = $result['StoppingInstances'][0]['CurrentState']['Name'];
      $instance = $this->entityTypeManager
        ->getStorage($entity->getEntityTypeId())
        ->load($entity->id());
      $instance->setInstanceState($current_state);
      $instance->save();

      $this->processOperationStatus($instance, 'stopped');
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSubmitEvent($entity);
      return TRUE;
    }
    catch (\Exception $e) {
      $this->handleException($e);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function rebootInstance(InstanceInterface $entity): bool {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->rebootInstances([
      'InstanceIds' => [
        $entity->getInstanceId(),
      ],
    ]);

    if (empty($result)) {
      $this->processOperationErrorStatus($entity, 'rebooted');
      return FALSE;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'rebooted remotely');
      return TRUE;
    }

    $this->processOperationStatus($entity, 'rebooted');
    $this->clearCacheValues($entity->getCacheTags());
    return TRUE;
  }

  /**
   * Gets the entity of this method.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity.
   */
  public function getEntity(): EntityInterface {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteInstance(InstanceInterface $entity): bool {
    $this->entity = $entity;

    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->terminateInstances([
      'InstanceIds' => [$entity->getInstanceId()],
    ]);

    if (empty($result)) {
      $this->processOperationErrorStatus($entity, 'deleted');
      return FALSE;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'deleted remotely');
      return TRUE;
    }

    if (isset($result['TerminatingInstances'][0]['InstanceId'])) {
      $entity->delete();

      $this->messenger->addStatus($this->getDeletionMessage());
      $this->logDeletionMessage();
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSubmitEvent($entity);
      return TRUE;
    }
    else {
      $this->processOperationErrorStatus($entity, 'deleted');
      return TRUE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConsoleOutputFromInstance(InstanceInterface $entity): string {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->getConsoleOutput(['InstanceId' => $entity->getInstanceId()]);

    $output = !empty($result) && !empty($result['Output'])
      ? base64_decode($result['Output'])
      : '';

    return !empty($output) ? $output : '';
  }

  /**
   * Helper method to load image entity.
   *
   * @param string $cloud_context
   *   Cloud context.
   * @param string $image_id
   *   Image id to load.
   *
   * @return null|\Drupal\aws_cloud\entity\ec2\ImageInterface
   *   NULL if not found else Image entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function loadNewImage(string $cloud_context, string $image_id): ?ImageInterface {
    $entity = NULL;
    $entities = $this->entityTypeManager
      ->getStorage('aws_cloud_image')
      ->loadByProperties([
        'cloud_context' => $cloud_context,
        'image_id' => $image_id,
      ]);

    if (!empty($entities)) {
      $entity = array_shift($entities);
    }
    return $entity;
  }

  /**
   * Helper function that loads an aws_cloud_elastic_ip entity.
   *
   * @param string $allocation_id
   *   The allocation ID to look up.
   * @param string $cloud_context
   *   The cloud context.
   *
   * @return \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface
   *   The loaded aws_cloud_elastic_ip entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getElasticIp(string $allocation_id, string $cloud_context): ElasticIpInterface {
    $elastic_ip = FALSE;
    $results = $this->entityTypeManager
      ->getStorage('aws_cloud_elastic_ip')
      ->loadByProperties([
        'allocation_id' => $allocation_id,
        'cloud_context' => $cloud_context,
      ]);
    if (count($results) === 1) {
      $elastic_ip = array_shift($results);
    }
    return $elastic_ip;
  }

  /**
   * {@inheritdoc}
   */
  public function createImageFromInstance(InstanceInterface $entity, array $param): bool {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $imageName = $param['imageName'];
    $noRebootFlg = $param['noRebootFlg'];

    $result = $this->ec2Service->createImage([
      'InstanceId' => $entity->getInstanceId(),
      'Name' => $imageName,
      'NoReboot' => $noRebootFlg,
    ]);

    if (!empty($entity) && !empty($result['ImageId'])) {

      // Call image update on this particular image.
      $this->ec2Service->updateImages([
        'ImageIds' => [
          $result['ImageId'],
        ],
      ]);

      $this->messenger->addStatus($this->t('The @type %label (%image_id) has been created.', [
        '@type' => $entity->getEntityType()->getSingularLabel(),
        '%label' => $entity->toLink($entity->label())->toString(),
        '%image_id' => $result['ImageId'],
      ]));
      $this->clearCacheValues($entity->getCacheTags());
      $image = $this->loadNewImage($entity->getCloudContext(), $result['ImageId']);
      if (!empty($image)) {
        $this->dispatchSubmitEvent($image);
      }
      return TRUE;
    }
    else {
      $this->processOperationErrorStatus($entity, 'created');
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function associateElasticIpForInstance(InstanceInterface $entity, array $param): bool {
    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $allocation_id = $param['allocationId'];
    $network_interface_id = $param['networkInterfaceId'];

    $result = $this->ec2Service->associateAddress([
      'AllocationId' => $allocation_id,
      'NetworkInterfaceId' => $network_interface_id,
    ]);

    if ($result !== NULL) {
      $this->ec2Service->updateElasticIps();
      $this->ec2Service->updateInstances();
      $this->ec2Service->updateNetworkInterfaces();

      $elastic_ip = $this->getElasticIp($allocation_id, $entity->getCloudContext());
      if ($elastic_ip !== FALSE) {
        $this->messenger->addStatus($this->t('Elastic IP @ip_address associated with @private_ip for instance: @instance', [
          '@ip_address' => $elastic_ip->getPublicIp(),
          '@instance' => $entity->getName(),
          '@private_ip' => $elastic_ip->getPrivateIpAddress(),
        ]));
        $this->clearCacheValues($entity->getCacheTags());
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
    else {
      $this->messenger->addError($this->t('Unable to associate Elastic IP.'));
      return FALSE;
    }
  }

  /**
   * Helper function to update AWS tags.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   */
  private function updateAwsTags(InstanceInterface $entity): void {
    $this->ec2Service->setCloudContext($entity->getCloudContext());

    $params = [
      'Resources' => [$entity->getInstanceId()],
    ];

    // Delete old tags.
    $this->ec2Service->deleteTags($params);

    foreach ($entity->getTags() ?: [] as $tag) {
      $params['Tags'][] = [
        'Key' => $tag['item_key'],
        'Value' => $tag['item_value'],
      ];
    }

    // Create Tags with different parameters for AWS and OpenStack.
    if (preg_match('[^aws_cloud]', $entity->getEntityTypeId()) === 1) {
      $this->ec2Service->createTags($params);
    }
    else {
      $this->ec2Service->createTags([
        'Resources' => [$entity->getInstanceId()],
        'Tags' => [
          ['Key' => 'Name', 'Value' => $entity->getName()],
        ],
      ]);
    }

  }

  /**
   * Helper function to update field tags.
   *
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   The Cloud service.
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param array $form_tags
   *   Form tags.
   * @param int|null $termination_timestamp
   *   Termination timestamp by UNIX seconds.
   * @param string|null $schedule
   *   Schedule.
   */
  private function updateTagsField(
    CloudServiceInterface $cloud_service,
    InstanceInterface $entity,
    array $form_tags,
    ?int $termination_timestamp,
    ?string $schedule
  ): void {
    usort($form_tags, static function ($a, $b) {
      return (int) $a['_weight'] - (int) $b['_weight'];
    });

    $module_name = $this->getModuleName($entity);
    $uid_key_name = $cloud_service->getTagCreatedByUid(
      'aws_cloud',
      $entity->getCloudContext(),
      Instance::TAG_LAUNCHED_BY_UID
    );

    $fixed_tags = [];
    $fixed_tags['Name'] = $entity->getName();
    $fixed_tags[$module_name . '_' . $uid_key_name] = $entity->getOwner() === NULL
    ? ''
    : $entity->getOwner()->id();

    $fixed_tags[$module_name . '_' . Instance::TAG_TERMINATION_TIMESTAMP] = $termination_timestamp === NULL
    ? ''
    : $termination_timestamp;

    $config = \Drupal::config('aws_cloud.settings');
    if ($config->get('aws_cloud_scheduler') === TRUE) {
      $fixed_tags['aws_cloud_scheduler_tag'] = $schedule;
    }

    $tags = [];
    foreach ($form_tags ?: [] as $form_tag) {
      $item_key = $form_tag['item_key'];
      if ($item_key === '') {
        continue;
      }

      // Skip special tags.
      if (strpos($item_key, 'aws:') === 0) {
        continue;
      }

      $item_value = $form_tag['item_value'];
      if (isset($fixed_tags[$item_key])) {
        $item_value = $fixed_tags[$item_key];
        unset($fixed_tags[$item_key]);
      }

      $tags[] = ['item_key' => $item_key, 'item_value' => $item_value];
    }

    foreach ($fixed_tags ?: [] as $item_key => $item_value) {
      $tags[] = ['item_key' => $item_key, 'item_value' => $item_value];
    }

    $entity->setTags($tags);
    $entity->save();
  }

  /**
   * Update the instance type.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param string|null $instance_type
   *   Instance type.
   * @param string|null $default_instance_type
   *   Default value of instance type.
   */
  private function updateInstanceType(
    InstanceInterface $entity,
    ?string $instance_type,
    ?string $default_instance_type
  ): void {
    // If the state of the instance is not stopped, do not update it.
    if ($entity->getInstanceState() !== 'stopped') {
      return;
    }

    // When executed from Form, $instance_type is NULL.
    // When executed from REST API, $instance_type is not NULL.
    // If it is run from Form and the instance type of $entity
    // is equal to the default instance type, the instance type
    // is not changed and no update is performed.
    if ($instance_type === NULL
      && $entity->getInstanceType() === $default_instance_type) {
      return;
    }

    $this->ec2Service->modifyInstanceAttribute([
      'InstanceId' => $entity->getInstanceId(),
      'InstanceType' => ['Value' => $instance_type === NULL ? $entity->getInstanceType() : $instance_type],
    ]);
  }

  /**
   * Get Security Group IDs by Names.
   *
   * @param array $group_names
   *   Array of group names.
   *
   * @return array
   *   Array of group names.
   */
  private function getSecurityGroupIdsByNames(array $group_names): array {
    $response = $this->ec2Service->describeSecurityGroups([
      'GroupNames' => $group_names,
    ]);

    $group_ids = [];
    foreach ($response['SecurityGroups'] ?: [] as $security_group) {
      $group_ids[] = $security_group['GroupId'];
    }

    return $group_ids;
  }

  /**
   * Update security group.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param string $security_group_delimiter
   *   Delimiter string for security groups.
   * @param array $default_security_groups
   *   Default value of security groups.
   * @param array $security_groups
   *   Security groups.
   */
  private function updateSecurityGroup(
    InstanceInterface $entity,
    string $security_group_delimiter,
    array $default_security_groups,
    array $security_groups
  ): void {
    if ($entity->getSecurityGroups() !== implode($security_group_delimiter, $default_security_groups)) {

      $this->ec2Service->modifyInstanceAttribute([
        'InstanceId' => $entity->getInstanceId(),
        'Groups' => $this->getSecurityGroupIdsByNames($security_groups),
      ]);
    }
  }

  /**
   * Update terminate protection.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param bool|null $termination_protection
   *   Termination protection flag.
   * @param bool|null $default_termination_protection
   *   Default value of termination protection flag.
   */
  private function updateTerminateProtection(
    InstanceInterface $entity,
    ?bool $termination_protection,
    ?bool $default_termination_protection
  ): void {
    if ($termination_protection === NULL
      && $entity->getTerminationProtection() === $default_termination_protection) {
      return;
    }

    $this->ec2Service->modifyInstanceAttribute([
      'InstanceId' => $entity->getInstanceId(),
      'DisableApiTermination' => [
        'Value' => $termination_protection === NULL ? $entity->getTerminationProtection() : $termination_protection,
      ],
    ]);
  }

  /**
   * Update IAM role.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   */
  private function updateIamRole(InstanceInterface $entity): void {
    if ($entity->getEntityTypeId() === 'aws_cloud_instance') {
      // Update IAM role.
      $associations_result = $this->ec2Service->describeIamInstanceProfileAssociations([
        'Filters' => [
          [
            'Name' => 'instance-id',
            'Values' => [$entity->getInstanceId()],
          ],
        ],
      ]);

      $associations = $associations_result['IamInstanceProfileAssociations'];
      if (empty($associations)) {
        if ($entity->getIamRole() !== NULL) {
          // Associate.
          $this->ec2Service->associateIamInstanceProfile([
            'InstanceId' => $entity->getInstanceId(),
            'IamInstanceProfile' => [
              'Arn' => $entity->getIamRole(),
            ],
          ]);
        }
      }
      else {
        if ($entity->getIamRole() === NULL) {
          // Disassociate.
          $this->ec2Service->disassociateIamInstanceProfile([
            'AssociationId' => $associations[0]['AssociationId'],
          ]);
        }
        elseif ($entity->getIamRole() !== $associations[0]['IamInstanceProfile']['Arn']) {
          // Disassociate.
          $this->ec2Service->disassociateIamInstanceProfile([
            'AssociationId' => $associations[0]['AssociationId'],
          ]);

          // Associate.
          $this->ec2Service->associateIamInstanceProfile([
            'InstanceId' => $entity->getInstanceId(),
            'IamInstanceProfile' => [
              'Arn' => $entity->getIamRole(),
            ],
          ]);
        }
      }
    }
  }

  /**
   * Update user data.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param string|null $user_data
   *   User data.
   * @param string|null $default_user_data
   *   Default value of user data.
   */
  private function updateUserData(InstanceInterface $entity, ?string $user_data, ?string $default_user_data): void {
    if ($entity->getInstanceState() !== 'stopped') {
      return;
    }

    if ($user_data === NULL
      && $entity->getUserData() === $default_user_data) {
      return;
    }

    $user_data_impl = $user_data === NULL ? $entity->getUserData() : $user_data;

    $this->ec2Service->modifyInstanceAttribute([
      'InstanceId' => $entity->getInstanceId(),
      'UserData' => ['Value' => $user_data_impl],
    ]);

    $url = Url::fromRoute(
      'entity.aws_cloud_instance.start_form', [
        'cloud_context' => $entity->getCloudContext(),
        'aws_cloud_instance' => $entity->id(),
      ]
    )->toString();

    $this->messenger->addStatus($this->t("The User Data of @type %label has been updated. Please <strong><em><a href='@start_instance_link'>start</a></em></strong> the @type to reflect the User Data.", [
      '@type' => $entity->getEntityType()->getSingularLabel(),
      '%label' => $entity->toLink($entity->label())->toString(),
      '@start_instance_link' => $url,
    ]));
    $this->logOperationMessage($entity, 'updated');
  }

  /**
   * If Elastic IP is specified and instance is stopped, attach it.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param string|null $new_elastic_ip
   *   New Elastic IP.
   * @param string|null $current_allocation_id
   *   Current Allocation ID.
   * @param string|null $current_association_id
   *   Current Association ID.
   */
  private function attachElasticIp(
    InstanceInterface $entity,
    ?string $new_elastic_ip,
    ?string $current_allocation_id,
    ?string $current_association_id
  ): void {
    if ($entity->getInstanceState() === 'stopped') {
      if (isset($new_elastic_ip) && (int) $new_elastic_ip !== -1) {
        $update_entities = FALSE;

        if (isset($current_allocation_id) && isset($current_association_id)) {
          // Instance already has allocation_id. Disassociate and reassociate.
          if ($current_allocation_id !== $new_elastic_ip) {
            $this->ec2Service->disassociateAddress([
              'AssociationId' => $current_association_id,
              'InstanceId' => $entity->getInstanceId(),
            ]);

            $this->ec2Service->associateAddress([
              'AllocationId' => $new_elastic_ip,
              'InstanceId' => $entity->getInstanceId(),
            ]);
            $update_entities = TRUE;
          }
        }
        else {
          // Instance does not have Elastic IP. Assign a new one.
          $this->ec2Service->associateAddress([
            'AllocationId' => $new_elastic_ip,
            'InstanceId' => $entity->getInstanceId(),
          ]);
          $update_entities = TRUE;
        }
        if ($update_entities === TRUE) {
          // Update the following entities if the Elastic IPs have changed.
          $this->ec2Service->updateElasticIps();
          $this->ec2Service->updateInstances();
          $this->ec2Service->updateNetworkInterfaces();
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function editInstanceBeforeSave(
    InstanceInterface $entity,
    ?int $termination_timestamp,
    array $security_groups_form_value,
    string $security_group_delimiter
  ): void {
    if ($termination_timestamp === NULL) {
      // Unset the termination timestamp.
      $entity->set('termination_timestamp', NULL);
    }

    $security_groups = array_values($security_groups_form_value);
    if ($security_groups === NULL) {
      $security_groups = [];
    }
    $entity->setSecurityGroups(implode($security_group_delimiter, $security_groups));
  }

  /**
   * Simulation of form submission handler for the 'save' action.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   */
  private function saveEntity(InstanceInterface $entity): void {
    $status = $entity->save();

    // Add a updated status message and the log.
    switch ($status) {
      case SAVED_NEW:
        $passive_operation = 'created';
        break;

      case SAVED_DELETED:
        $passive_operation = 'deleted';
        break;

      default:
      case SAVED_UPDATED:
        $passive_operation = 'updated';
    }
    $this->processOperationStatus($entity, $passive_operation);

    // Clear block and menu cache.
    $this->clearCacheValues($entity->getCacheTags());
  }

  /**
   * {@inheritdoc}
   */
  public function editInstanceAfterSave(
    CloudServiceInterface $cloud_service,
    InstanceInterface $entity,
    array $form_tags,
    ?int $termination_timestamp,
    ?string $schedule,
    ?string $instance_type,
    ?string $default_instance_type,
    array $security_groups_form_value,
    string $security_group_delimiter,
    array $default_security_groups,
    ?bool $termination_protection,
    ?bool $default_termination_protection,
    ?string $user_data,
    ?string $default_user_data,
    ?string $new_elastic_ip,
    ?string $current_allocation_id,
    ?string $current_association_id
  ): void {
    $this->updateTagsField(
      $cloud_service,
      $entity,
      $form_tags,
      $termination_timestamp,
      $schedule
    );

    $this->ec2Service->setCloudContext($entity->getCloudContext());

    $this->updateAwsTags($entity);

    // Update the instance type.
    $this->updateInstanceType($entity, $instance_type, $default_instance_type);

    // Update security group.
    $security_groups = array_values($security_groups_form_value);
    if ($security_groups === NULL) {
      $security_groups = [];
    }
    $this->updateSecurityGroup(
      $entity,
      $security_group_delimiter,
      $default_security_groups,
      $security_groups
    );

    // Update terminate protection.
    $this->updateTerminateProtection($entity, $termination_protection, $default_termination_protection);

    // Update IAM role.
    $this->updateIamRole($entity);

    // Update user data.
    $this->updateUserData($entity, $user_data, $default_user_data);

    // If Elastic IP is specified and instance is stopped, attach it.
    $this->attachElasticIp(
      $entity,
      $new_elastic_ip,
      $current_allocation_id,
      $current_association_id
    );

    $this->clearCacheValues($entity->getCacheTags());
    $this->dispatchSaveEvent($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function editInstance(InstanceInterface $entity, array $param): bool {
    $default_instance_type = $entity->getInstanceType();
    $default_security_groups = explode(InstanceEditForm::SECURITY_GROUP_DELIMITER, $entity->getSecurityGroups());
    $default_termination_protection = $entity->getTerminationProtection();
    $default_user_data = $entity->getUserData();

    $this->editInstanceBeforeSave(
      $entity,
      $param['termination_timestamp'],
      $param['security_groups'],
      InstanceEditForm::SECURITY_GROUP_DELIMITER
    );

    // Save entity.
    if ($param['instance_type'] !== NULL) {
      $entity->setInstanceType($param['instance_type']);
    }
    if ($param['termination_protection'] !== NULL) {
      $entity->setTerminationProtection($param['termination_protection']);
    }
    if ($param['user_data'] !== NULL) {
      $entity->setUserData($param['user_data']);
    }
    $this->saveEntity($entity);

    $config = \Drupal::config('aws_cloud.settings');
    $this->editInstanceAfterSave(
      $this->cloudService,
      $entity,
      $param['tags'],
      $param['termination_timestamp'],
      $config->get('aws_cloud_scheduler') === TRUE ? $param['schedule'] : NULL,
      $param['instance_type'],
      $default_instance_type,
      $param['security_groups'],
      InstanceEditForm::SECURITY_GROUP_DELIMITER,
      $default_security_groups,
      $param['termination_protection'],
      $default_termination_protection,
      $param['user_data'],
      $default_user_data,
      $param['new_elastic_ip'],
      $param['current_allocation_id'],
      $param['current_association_id']
    );
    return TRUE;
  }

}
