<?php

namespace Drupal\aws_cloud\Service\Ec2;

use Drupal\aws_cloud\Entity\Ec2\InstanceInterface;
use Drupal\cloud\Service\CloudServiceInterface;

/**
 * The Interface for Ec2OperationsService.
 */
interface Ec2OperationsServiceInterface {

  /**
   * Start AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function startInstance(InstanceInterface $entity): bool;

  /**
   * Stop AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function stopInstance(InstanceInterface $entity): bool;

  /**
   * Reboot AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function rebootInstance(InstanceInterface $entity): bool;

  /**
   * Delete AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteInstance(InstanceInterface $entity): bool;

  /**
   * Get console output from AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   *
   * @return string
   *   Console output.
   */
  public function getConsoleOutputFromInstance(InstanceInterface $entity): string;

  /**
   * Create image from AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param array $param
   *   Parameter.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createImageFromInstance(InstanceInterface $entity, array $param): bool;

  /**
   * Associate an Elastic IP for AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param array $param
   *   Parameter.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function associateElasticIpForInstance(InstanceInterface $entity, array $param): bool;

  /**
   * Edit AWS Cloud instance before save method.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param int|null $termination_timestamp
   *   Termination timestamp by UNIX seconds.
   * @param array $security_groups_form_value
   *   Security groups value on form.
   * @param string $security_group_delimiter
   *   Delimiter string for security groups.
   */
  public function editInstanceBeforeSave(
    InstanceInterface $entity,
    ?int $termination_timestamp,
    array $security_groups_form_value,
    string $security_group_delimiter
  ): void;

  /**
   * Edit AWS Cloud instance after save method.
   *
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   The Cloud service.
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param array $form_tags
   *   Form tags.
   * @param int|null $termination_timestamp
   *   Termination timestamp by UNIX seconds.
   * @param string|null $schedule
   *   Schedule.
   * @param string|null $instance_type
   *   Instance type.
   * @param string|null $default_instance_type
   *   Default value of instance type.
   * @param array $security_groups_form_value
   *   Security groups value on form.
   * @param string $security_group_delimiter
   *   Delimiter string for security groups.
   * @param array $default_security_groups
   *   Default value of security groups.
   * @param bool|null $termination_protection
   *   Termination protection flag.
   * @param bool|null $default_termination_protection
   *   Default value of termination protection flag.
   * @param string|null $user_data
   *   User data.
   * @param string|null $default_user_data
   *   Default value of user data.
   * @param string|null $new_elastic_ip
   *   New Elastic IP.
   * @param string|null $current_allocation_id
   *   Current Allocation ID.
   * @param string|null $current_association_id
   *   Current Association ID.
   */
  public function editInstanceAfterSave(
    CloudServiceInterface $cloud_service,
    InstanceInterface $entity,
    array $form_tags,
    ?int $termination_timestamp,
    ?string $schedule,
    ?string $instance_type,
    ?string $default_instance_type,
    array $security_groups_form_value,
    string $security_group_delimiter,
    array $default_security_groups,
    ?bool $termination_protection,
    ?bool $default_termination_protection,
    ?string $user_data,
    ?string $default_user_data,
    ?string $new_elastic_ip,
    ?string $current_allocation_id,
    ?string $current_association_id
  ): void;

  /**
   * Edit AWS Cloud instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The AWS Cloud instance entity.
   * @param array $param
   *   Parameter.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editInstance(InstanceInterface $entity, array $param): bool;

}
