<?php

namespace Drupal\k8s\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\k8s\Service\K8sServiceInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates YAML URL fields.
 */
class YamlUrlConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * The K8s Service.
   *
   * @var \Drupal\k8s\Service\K8sServiceInterface
   */
  protected $k8sService;

  /**
   * Constructs a new constraint validator.
   *
   * @param \Drupal\k8s\Service\K8sServiceInterface $k8s_service
   *   The K8s Service.
   */
  public function __construct(K8sServiceInterface $k8s_service) {
    $this->k8sService = $k8s_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('k8s')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($entity, Constraint $constraint): void {

    // Only perform validations for K8s bundles.
    if ($entity->bundle() !== 'k8s') {
      return;
    }

    $yaml_url = $entity->get('field_yaml_url')->uri;
    $detail = $entity->get('field_detail')->value;
    $source_type = $entity->get('field_source_type')->value;

    if (str_contains($source_type, 'git')) {
      if (empty($yaml_url)) {
        $this->context
          ->buildViolation($constraint->requiredGitUrl)
          ->atPath('field_yaml_url')
          ->addViolation();
        return;
      }

      if (strpos($yaml_url, '.git') !== strlen($yaml_url) - 4) {
        $this->context
          ->buildViolation($constraint->invalidGitUrl)
          ->atPath('field_yaml_url')
          ->addViolation();
        return;
      }

      $client = \Drupal::httpClient();
      $url = substr($yaml_url, 0, strrpos($yaml_url, '/'));
      try {
        $client->get($url);
      }
      catch (RequestException $e) {
        $this->context
          ->buildViolation($constraint->unreachableGitUrl)
          ->atPath('field_yaml_url')
          ->addViolation();
      }
      return;
    }

    if (empty($yaml_url) && empty($detail)) {
      $this->context
        ->buildViolation($constraint->requiredYamlUrlOrDetail)
        ->atPath('field_yaml_url')
        ->addViolation();
      return;
    }

    if (!empty($yaml_url) && !empty($detail)) {
      $this->context
        ->buildViolation($constraint->prohibitYamlUrlAndDetail)
        ->atPath('field_yaml_url')
        ->addViolation();
      return;
    }

    if (empty($yaml_url)) {
      return;
    }

    $content = file_get_contents($yaml_url);
    if (empty($content)) {
      $this->context
        ->buildViolation($constraint->invalidYamlUrl)
        ->atPath('field_yaml_url')
        ->addViolation();
      return;
    }

    $templates = $this->k8sService->supportedCloudLaunchTemplates();
    try {
      $yamls = $this->k8sService->decodeMultipleDocYaml($content);
      foreach ($yamls ?: [] as $yaml) {
        if (!is_array($yaml)) {
          $this->context
            ->buildViolation($constraint->invalidYamlFormat)
            ->atPath('field_yaml_url')
            ->addViolation();
          return;
        }

        if (empty($yaml['kind'])) {
          $this->context
            ->buildViolation($constraint->noKindFound)
            ->atPath('field_yaml_url')
            ->addViolation();
          return;
        }

        $kind = $yaml['kind'];
        $object = array_search($kind, $templates);
        if (empty($object)) {
          $this->context
            ->buildViolation($constraint->unsupportedObjectType, ['%kind' => $kind])
            ->atPath('field_yaml_url')
            ->addViolation();
          return;
        }
      }
    }
    catch (\Exception $e) {
      $this->context
        ->buildViolation($constraint->invalidYaml . $e->getMessage())
        ->atPath('field_yaml_url')
        ->addViolation();
    }
  }

}
