<?php

namespace Drupal\cloud\Plugin\cloud\launch_template;

use Drupal\cloud\Entity\CloudLaunchTemplateInterface;
use Drupal\cloud\Plugin\cloud\CloudPluginManager;
use Drupal\Component\Plugin\Discovery\DiscoveryInterface;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Discovery\ContainerDerivativeDiscoveryDecorator;
use Drupal\Core\Plugin\Discovery\YamlDiscovery;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Routing\Route;

/**
 * Provides the default cloud_launch_template_plugin manager.
 */
class CloudLaunchTemplatePluginManager extends CloudPluginManager implements CloudLaunchTemplatePluginManagerInterface, ContainerInjectionInterface {

  /**
   * Provides default values for all cloud_launch_template_plugin plugins.
   *
   * @var array
   */
  protected $defaults = [
    'id' => 'cloud_launch_template',
    'entity_type' => 'cloud_launch_template',
  ];

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * Constructs a new CloudLaunchTemplatePluginManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack object.
   */
  public function __construct(
    \Traversable $namespaces,
    ModuleHandlerInterface $module_handler,
    CacheBackendInterface $cache_backend,
    RequestStack $request_stack
  ) {

    parent::__construct('Plugin\cloud\launch_template', $namespaces, $module_handler);

    // Add more services as required.
    $this->moduleHandler = $module_handler;
    $this->setCacheBackend($cache_backend, 'cloud_launch_template_plugin', ['cloud_launch_template_plugin']);
    $this->requestStack = $request_stack;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Instance of ContainerInterface.
   *
   * @return CloudLaunchTemplatePluginManager
   *   return created object.
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('container.namespaces'),
    $container->get('module_handler'),
    $container->get('cache.discovery'),
    $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getDiscovery(): DiscoveryInterface {
    if (!isset($this->discovery)) {
      $this->discovery = new YamlDiscovery('cloud.launch.template.plugin', $this->moduleHandler->getModuleDirectories());
      $this->discovery->addTranslatableProperty('label', 'label_context');
      $this->discovery = new ContainerDerivativeDiscoveryDecorator($this->discovery);
    }
    return $this->discovery;
  }

  /**
   * {@inheritdoc}
   */
  public function processDefinition(&$definition, $plugin_id): void {
    parent::processDefinition($definition, $plugin_id);

    if (empty($definition['id'])) {
      throw new PluginException(sprintf('CloudLaunchTemplatePlugin plugin property (%s) definition "is" is required.', $plugin_id));
    }

    if (empty($definition['entity_bundle'])) {
      throw new PluginException(sprintf('entity_bundle property is required for (%s)', $plugin_id));
    }

    if (empty($definition['cloud_context'])) {
      throw new PluginException(sprintf('cloud_context property is required for (%s)', $plugin_id));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function loadPluginVariant($cloud_context): ?CloudLaunchTemplatePluginInterface {
    $plugin = NULL;
    foreach ($this->getDefinitions() ?: [] as $key => $definition) {
      if ($definition['cloud_context'] === $cloud_context) {
        $plugin = $this->createInstance($key);
        break;
      }
    }

    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function launch(CloudLaunchTemplateInterface $cloud_launch_template, FormStateInterface $form_state = NULL): array {
    $plugin = $this->loadPluginVariant($cloud_launch_template->getCloudContext());
    if ($plugin === FALSE) {
      $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', [
        '%cloud_context' => $cloud_launch_template->getCloudContext(),
      ]));

      return [
        'route_name' => 'entity.cloud_launch_template.canonical',
        'params' => [
          'cloud_launch_template' => $cloud_launch_template->id(),
          'cloud_context' => $cloud_launch_template->getCloudContext(),
        ],
      ];
    }

    return $plugin->launch($cloud_launch_template, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function buildListHeader($cloud_context): array {
    if (is_array($cloud_context)) {
      foreach ($cloud_context ?: [] as $cloud_context_value) {
        $plugin = $this->loadPluginVariant($cloud_context_value);
      }
    }
    else {
      $plugin = $this->loadPluginVariant($cloud_context);
    }

    if (empty($plugin)) {
      $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', [
        '%cloud_context' => $cloud_context,
      ]));
      return [];
    }

    return $plugin->buildListHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildListRow(CloudLaunchTemplateInterface $entity): array {
    $plugin = $this->loadPluginVariant($entity->getCloudContext());
    if (empty($plugin)) {
      $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', [
        '%cloud_context' => $entity->getCloudContext(),
      ]));
      return [];
    }

    return $plugin->buildListRow($entity);
  }

  /**
   * Update cloud launch template list.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param string $cloud_server_template_type
   *   The cloud launch template type.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   An array of created object.
   */
  public function updateCloudLaunchTemplateList($cloud_context = '', $cloud_server_template_type = ''): RedirectResponse {
    if (empty($cloud_context)) {
      return $this->updateAllCloudLaunchTemplateList($cloud_server_template_type);
    }

    $referer = $this->requestStack->getCurrentRequest()->headers->get('referer');

    $plugin = $this->loadPluginVariant($cloud_context);

    if ($plugin === FALSE) {
      $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', ['%cloud_context' => $cloud_context]));
    }
    elseif (!method_exists($plugin, 'updateCloudLaunchTemplateList')) {
      $this->messenger->addStatus($this->t('Unnecessary to update launch templates.'));
    }
    else {
      $updated = $plugin->updateCloudLaunchTemplateList($cloud_context);

      if ($updated !== FALSE) {
        $this->messenger->addStatus($this->t('Updated launch templates.'));
      }
      else {
        $this->messenger->addError($this->t('Unable to update launch templates.'));
      }
    }
    return new RedirectResponse($referer);
  }

  /**
   * Update all cloud launch template list.
   *
   * @param string $cloud_server_template_type
   *   The cloud launch template type.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   An array of created object.
   */
  public function updateAllCloudLaunchTemplateList($cloud_server_template_type): RedirectResponse {

    $referer = $this->requestStack->getCurrentRequest()->headers->get('referer');

    $cloud_configs = \Drupal::entityTypeManager()
      ->getStorage('cloud_config')
      ->loadByProperties([
        'type' => $cloud_server_template_type,
      ]);

    foreach ($cloud_configs ?: [] as $cloud_config) {
      $cloud_context = $cloud_config->getCloudContext();
      $plugin = $this->loadPluginVariant($cloud_context);

      if ($plugin === FALSE) {
        $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', ['%cloud_context' => $cloud_context]));
      }
      elseif (!method_exists($plugin, 'updateAllCloudLaunchTemplateList')) {
        $this->messenger->addStatus($this->t('Unnecessary to update launch templates.'));
      }
      else {
        $updated = $plugin->updateAllCloudLaunchTemplateList($cloud_context);

        if ($updated !== FALSE) {
          $this->messenger->addStatus($this->t('Updated launch templates.'));
        }
        else {
          $this->messenger->addError($this->t('Unable to update launch templates.'));
        }
      }
    }
    return new RedirectResponse($referer);
  }

  /**
   * {@inheritdoc}
   */
  public function buildLaunchForm(CloudLaunchTemplateInterface $cloud_launch_template, array &$form, FormStateInterface $form_state): void {
    $plugin = $this->loadPluginVariant($cloud_launch_template->getCloudContext());
    if ($plugin === FALSE) {
      $this->messenger->addStatus($this->t('Cannot load launch template plugin: %cloud_context', [
        '%cloud_context' => $cloud_launch_template->getCloudContext(),
      ]));
      return;
    }

    if (method_exists($plugin, 'buildLaunchForm')) {
      $plugin->buildLaunchForm($form, $form_state);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function access(string $cloud_context, AccountInterface $account, Route $route): AccessResultInterface {
    return $this->loadPluginVariant($cloud_context)->access($cloud_context, $account, $route);
  }

}
